package mca.client.render;

import mca.client.model.ModelVillagerMCA;
import mca.entity.EntityVillagerMCA;
import mca.enums.EnumAgeState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.model.ModelBiped;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.entity.RenderBiped;
import net.minecraft.client.renderer.entity.RenderManager;
import net.minecraft.client.renderer.entity.layers.LayerBipedArmor;
import net.minecraft.client.renderer.entity.layers.LayerHeldItem;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.util.ResourceLocation;
import org.lwjgl.opengl.GL11;

public class RenderVillagerMCA<T extends EntityVillagerMCA> extends RenderBiped<EntityVillagerMCA> {
    private static final ResourceLocation gui = new ResourceLocation("mca:textures/gui.png");
    private static final float LABEL_SCALE = 0.027F;

    public RenderVillagerMCA(RenderManager manager) {
        super(manager, new ModelVillagerMCA(), 0.5F);
        this.func_177094_a(new LayerBipedArmor(this));
        this.func_177094_a(new LayerHeldItem(this));
    }

    @Override
    protected void func_77041_b(EntityVillagerMCA villager, float partialTickTime) {
        if (villager.func_70631_g_()) {
            float scaleForAge = EnumAgeState.byId(villager.get(EntityVillagerMCA.AGE_STATE)).getScaleForAge();
            GlStateManager.func_179152_a(scaleForAge, scaleForAge, scaleForAge);
        }

        if (villager.func_184218_aH()) {
            GlStateManager.func_179137_b(0, 0.5, 0);
        }
    }

    @Override
    public void func_177067_a(EntityVillagerMCA entity, double x, double y, double z) {
        float modY = entity.isSleeping() ? -1.5F : 0;
        super.func_177067_a(entity, x, y + modY, z);

        if (func_177070_b(entity)) {
            if (entity.func_110143_aJ() < entity.func_110138_aP()) {
                renderHealth(entity, x, y + modY, z, (int) entity.func_110143_aJ(), (int) entity.func_110138_aP());
            }

            if (entity.getCurrentActivity() != null) {
                double d0 = entity.func_70068_e(this.field_76990_c.field_78734_h);
                float f = entity.func_70093_af() ? NAME_TAG_RANGE_SNEAK : NAME_TAG_RANGE;
                if (d0 < (double) (f * f)) {
                    this.func_188296_a(entity, x, y - 0.25F + modY , z, "(" + entity.getCurrentActivity() + ")", d0);
                }
            }
        }
    }

    private void renderHealth(EntityVillagerMCA villager, double posX, double posY, double posZ, int currentHealth, int maxHealth) {
        final int redHeartU = 80;
        final int darkHeartU = 96;
        int heartsDrawn = 0;

        float maxHealthF = Math.round((float)maxHealth / 2.0F);
        float currentHealthF = Math.round((float)currentHealth / 2.0F);
        int heartsMax = Math.round((maxHealthF / maxHealthF) * 10.0F);
        int heartsToDraw = Math.round((currentHealthF / maxHealthF) * 10.0F);

        for (int i = 0; i < heartsMax; i++) {
            int heartU = i < heartsToDraw ? redHeartU : darkHeartU;
            heartsDrawn++;

            GL11.glPushMatrix();{
                GL11.glTranslatef((float) posX + 0.0F, (float) posY + villager.field_70131_O + 1.0F, (float) posZ);
                GL11.glRotatef(-field_76990_c.field_78735_i, 0.0F, 1.0F, 0.0F);
                GL11.glRotatef(field_76990_c.field_78732_j, 1.0F, 0.0F, 0.0F);
                GL11.glScalef(-LABEL_SCALE, -LABEL_SCALE, LABEL_SCALE);
                GL11.glDisable(GL11.GL_LIGHTING);
                GL11.glTranslatef(-2.0F, 2.0F, -2.0F);
                drawTexturedRectangle(gui, (int)posX + (heartsDrawn * 8) - 45, (int)posY - 4, heartU, 0, 16, 16);
            }
            GL11.glPopMatrix();
            GL11.glDepthMask(true);
            GL11.glEnable(GL11.GL_LIGHTING);
        }
    }

    @Override
    protected ResourceLocation func_110775_a(EntityVillagerMCA villager) {
        return villager.getTextureResourceLocation();
    }

    @Override
    protected boolean func_177070_b(EntityVillagerMCA entity) {
        float distance = Minecraft.func_71410_x().field_71439_g.func_70032_d(entity);
        return distance < 5F;
    }

    public static void drawTexturedRectangle(ResourceLocation texture, int x, int y, int u, int v, int width, int height)
    {
        Minecraft.func_71410_x().field_71446_o.func_110577_a(texture);

        float f = 0.00390625F;
        float f1 = 0.00390625F;

        final Tessellator tessellator = Tessellator.func_178181_a();
        BufferBuilder buffer = tessellator.func_178180_c();

        buffer.func_181668_a(7, DefaultVertexFormats.field_181707_g);
        buffer.func_181662_b(x + 0, y + height, 0.0D).func_187315_a((u + 0) * f, ((v + height) * f1)).func_181675_d();
        buffer.func_181662_b(x + width, y + height, 0.0D).func_187315_a((u + width) * f, ((v + height) * f1)).func_181675_d();
        buffer.func_181662_b(x + width, y + 0,	0.0D).func_187315_a((u + width) * f, ((v + 0) * f1)).func_181675_d();
        buffer.func_181662_b(x + 0, y + 0, 0.0D).func_187315_a((u + 0) * f, ((v + 0) * f1)).func_181675_d();
        tessellator.func_78381_a();
    }

    @Override
    protected void func_77039_a(EntityVillagerMCA entityLiving, double x, double y, double z) {
        if (entityLiving.func_70089_S() && entityLiving.isSleeping()) {
            super.func_77039_a(entityLiving, x + (double)entityLiving.renderOffsetX, y + (double)entityLiving.renderOffsetY, z + (double)entityLiving.renderOffsetZ);
        } else {
            super.func_77039_a(entityLiving, x, y, z);
        }
    }

    @Override
    protected void func_77043_a(EntityVillagerMCA entityLiving, float p_77043_2_, float rotationYaw, float partialTicks) {
        if (entityLiving.isSleeping()) {
            GlStateManager.func_179114_b(entityLiving.getBedOrientationInDegrees(), 0.0F, 1.0F, 0.0F);
            GlStateManager.func_179114_b(this.func_77037_a(entityLiving), 0.0F, 0.0F, 1.0F);
            GlStateManager.func_179114_b(270.0F, 0.0F, 1.0F, 0.0F);
            rotationYaw = 180.0f;
        }
        super.func_77043_a(entityLiving, p_77043_2_, rotationYaw, partialTicks);
    }
}
