package mca.command;

import com.google.common.base.Optional;
import mca.core.Constants;
import mca.core.MCA;
import mca.core.minecraft.ItemsMCA;
import mca.entity.EntityGrimReaper;
import mca.entity.EntityVillagerMCA;
import mca.entity.data.PlayerSaveData;
import mca.items.ItemBaby;
import mca.util.Util;
import net.minecraft.command.CommandBase;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommandSender;
import net.minecraft.command.WrongUsageException;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.text.TextComponentString;
import java.util.Arrays;
import java.util.UUID;


import java.util.Arrays;
import java.util.UUID;

public class CommandAdminMCA extends CommandBase {
    @Override
    public String func_71517_b() {
        return "mca-admin";
    }

    @Override
    public String func_71518_a(ICommandSender commandSender) {
        return "/mca-admin <subcommand> <arguments>";
    }

    @Override
    public void func_184881_a(MinecraftServer server, ICommandSender commandSender, String[] input) throws CommandException {
        try {
            if (!MCA.getConfig().enableAdminCommands) {
                sendMessage(commandSender, "MCA admin commands have been disabled by the server administrator.");
                return;
            }

            if (input.length == 0) {
                throw new WrongUsageException("");
            }

            final EntityPlayer player = (EntityPlayer) commandSender;
            String subcommand = input[0].toLowerCase();
            String[] arguments = Arrays.copyOfRange(input, 1, input.length);
            MCA.getLog().info(player.func_70005_c_() + " entered debug command " + Arrays.toString(input));

            switch (subcommand) {
                case "help": displayHelp(commandSender); break;
                case "ffh": forceFullHearts(player); break;
                case "fbg": forceBabyGrow(player); break;
                case "fcg": forceChildGrow(player); break;
                case "clv": clearLoadedVillagers(player);break;
                case "inh": incrementHearts(player); break;
                case "deh": decrementHearts(player); break;
                case "sgr": spawnGrimReaper(player); break;
                case "kgr": killGrimReaper(player); break;
                case "dpd": dumpPlayerData(player); break;
                case "rvd": resetVillagerData(player, arguments); break;
                case "rpd": resetPlayerData(player, arguments); break;
                case "cve": clearVillagerEditors(player); break;
                default: throw new WrongUsageException("");
            }
        } catch (ClassCastException e) {
            throw new CommandException("MCA commands cannot be used through rcon.");
        } catch (WrongUsageException e) {
            throw new CommandException("Your command was invalid or improperly formatted. Usage: " + func_71518_a(commandSender));
        }
    }

    private void forceFullHearts(EntityPlayer player) {
        for (Entity entity : player.field_70170_p.field_72996_f) {
            if (entity instanceof EntityVillagerMCA) {
                EntityVillagerMCA villager = (EntityVillagerMCA) entity;
                villager.getPlayerHistoryFor(player.func_110124_au()).setHearts(100);
            }
        }
        sendMessage(player, Constants.Color.GREEN + "Forced full hearts on all villagers.");
    }

    private void forceBabyGrow(EntityPlayer player) {
        for (ItemStack stack : player.field_71071_by.field_70462_a) {
            if (stack.func_77973_b() instanceof ItemBaby) {
                stack.func_77978_p().func_74768_a("age", MCA.getConfig().babyGrowUpTime);
            }
        }
        sendMessage(player, Constants.Color.GREEN + "Forced any held babies to grow up age.");
    }

    private void forceChildGrow(EntityPlayer player) {
        player.field_70170_p.field_72996_f.stream()
                .filter(e -> e instanceof EntityVillagerMCA && ((EntityVillagerMCA)e).func_70631_g_())
                .forEach(e -> ((EntityVillagerMCA) e).func_110195_a(999999));
        sendMessage(player, Constants.Color.GREEN + "Forced any children to grow to adults.");
    }

    private void clearLoadedVillagers(EntityPlayer player) {
        int n = 0;
        for (Entity entity : player.field_70170_p.field_72996_f) {
            if (entity instanceof EntityVillagerMCA) {
                entity.func_70106_y();
                ++n;
            }
        }
        sendMessage(player, Constants.Color.GREEN + "Cleared " + n + " villagers from the world.");
    }

    private void incrementHearts(EntityPlayer player) {
        for (Entity entity : player.field_70170_p.field_72996_f) {
            if (entity instanceof EntityVillagerMCA) {
                EntityVillagerMCA villager = (EntityVillagerMCA) entity;
                villager.getPlayerHistoryFor(player.func_110124_au()).changeHearts(10);
            }
        }
        sendMessage(player, Constants.Color.GREEN + "Increased hearts for all villagers by 10.");
    }

    private void decrementHearts(EntityPlayer player) {
        for (Entity entity : player.field_70170_p.field_72996_f) {
            if (entity instanceof EntityVillagerMCA) {
                EntityVillagerMCA villager = (EntityVillagerMCA) entity;
                villager.getPlayerHistoryFor(player.func_110124_au()).changeHearts(-10);
            }
        }
        sendMessage(player, Constants.Color.GREEN + "Decreased hearts for all villagers by 10.");
    }

    private void spawnGrimReaper(EntityPlayer player) {
        EntityGrimReaper reaper = new EntityGrimReaper(player.field_70170_p);
        reaper.func_70107_b(player.field_70165_t, player.field_70163_u, player.field_70161_v);
        player.field_70170_p.func_72838_d(reaper);
    }

    private void killGrimReaper(EntityPlayer player) {
        player.field_70170_p.field_72996_f.stream().filter((e) -> e instanceof EntityGrimReaper).forEach((e) -> e.func_70106_y());
    }

    private void dumpPlayerData(EntityPlayer player) {
        PlayerSaveData.get(player).dump(player);
    }

    private void resetVillagerData(EntityPlayer sender, String[] arguments) {
        Optional<EntityVillagerMCA> target = Util.getEntityByUUID(sender.field_70170_p, UUID.fromString(arguments[0]), EntityVillagerMCA.class);
        if (!target.isPresent()) {
            sendMessage(sender, "Target villager was not found.");
        } else {
            target.get().reset();
            sendMessage(sender, target.get().func_145748_c_().func_150260_c() + " has been reset successfully.");
        }
    }

    private void resetPlayerData(EntityPlayer sender, String[] arguments) {
        Optional<Entity> target = com.google.common.base.Optional.fromJavaUtil(sender.field_70170_p.field_72996_f.stream()
                .filter(e -> e instanceof EntityPlayer && e.func_70005_c_().equals(arguments[0]))
                .findFirst());
        if (!target.isPresent()) {
            sendMessage(sender, "Player not found on the server.");
        } else {
            PlayerSaveData.get((EntityPlayer)target.get()).reset();
            sendMessage(sender, "Player data for " + target.get().func_70005_c_() + " has been reset successfully.");
            sendMessage(target.get(), "Your player data has been reset by " + sender.func_70005_c_() + ".");
        }
    }

    private void clearVillagerEditors(EntityPlayer sender) {
        ItemStack editorStack = new ItemStack(ItemsMCA.VILLAGER_EDITOR);
        sender.field_70170_p.field_73010_i.stream().filter(p -> p.field_71071_by.func_70431_c(editorStack)).forEach(p -> {
            int i = 0;
            while (i < p.field_71071_by.func_70302_i_() - 1) {
                if (p.field_71071_by.func_70301_a(i).func_77973_b() == ItemsMCA.VILLAGER_EDITOR) {
                    p.field_71071_by.func_70299_a(i, ItemStack.field_190927_a);
                }
                i++;
            }
        });
        sendMessage(sender, "All villager editors cleared from inventories.");
    }

    @Override
    public int func_82362_a() {
        return 4;
    }

    private void sendMessage(ICommandSender commandSender, String message) {
        commandSender.func_145747_a(new TextComponentString(Constants.Color.GOLD + "[MCA] " + Constants.Format.RESET + message));
    }

    private void sendMessage(ICommandSender commandSender, String message, boolean noPrefix) {
        if (noPrefix) {
            commandSender.func_145747_a(new TextComponentString(message));
        } else {
            sendMessage(commandSender, message);
        }
    }

    private void displayHelp(ICommandSender commandSender) {
        sendMessage(commandSender, Constants.Color.DARKRED + "--- " + Constants.Color.GOLD + "OP COMMANDS" + Constants.Color.DARKRED + " ---", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin ffh " + Constants.Color.GOLD + " - Force all hearts on all villagers.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin fbg " + Constants.Color.GOLD + " - Force your baby to grow up.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin fcg " + Constants.Color.GOLD + " - Force nearby children to grow.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin clv " + Constants.Color.GOLD + " - Clear all loaded villagers. " + Constants.Color.RED + "(IRREVERSABLE)", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin inh " + Constants.Color.GOLD + " - Increase hearts by 10.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin deh " + Constants.Color.GOLD + " - Decrease hearts by 10.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin kgr " + Constants.Color.GOLD + " - Kill all Grim Reapers in the world.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin dpd " + Constants.Color.GOLD + " - Dumps player data to chat.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin rvd <UUID>" + Constants.Color.GOLD + " - Resets the given villager.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin rpd <PlayerName>" + Constants.Color.GOLD + " - Resets the given player's MCA data.", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin cve" + Constants.Color.GOLD + " - Remove all villager editors from the game.", true);

        sendMessage(commandSender, Constants.Color.DARKRED + "--- " + Constants.Color.GOLD + "GLOBAL COMMANDS" + Constants.Color.DARKRED + " ---", true);
        sendMessage(commandSender, Constants.Color.WHITE + " /mca-admin help " + Constants.Color.GOLD + " - Shows this list of commands.", true);
    }
}
