package mca.core.minecraft;

import com.google.common.base.Optional;
import mca.core.MCA;
import mca.entity.EntityVillagerMCA;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.village.Village;
import net.minecraft.world.World;
import java.util.List;


import java.util.List;

public class VillageHelper {

    public static void tick(World world) {
        world.func_175714_ae().func_75540_b().forEach(v -> spawnGuards(world, v));
    }

    public static void forceSpawnGuards(EntityPlayerMP player) {
        Village nearestVillage = player.field_70170_p.func_175714_ae().func_176056_a(player.func_180425_c(), 100);
        spawnGuards(player.field_70170_p, nearestVillage);
    }

    public static void forceRaid(EntityPlayerMP player) {
        Village nearestVillage = player.field_70170_p.func_175714_ae().func_176056_a(player.func_180425_c(), 100);
        startRaid(player.field_70170_p, nearestVillage);
    }

    private static void spawnGuards(World world, Village village) {
        int guardCapacity = village.func_75562_e() / MCA.getConfig().guardSpawnRate;
        int guards = 0;

        // Grab all villagers in the area
        List<EntityVillagerMCA> list = world.func_72872_a(EntityVillagerMCA.class,
                new AxisAlignedBB((double) (village.func_180608_a().func_177958_n() - village.func_75568_b()),
                        (double) (village.func_180608_a().func_177956_o() - 4),
                        (double) (village.func_180608_a().func_177952_p() - village.func_75568_b()),
                        (double) (village.func_180608_a().func_177958_n() + village.func_75568_b()),
                        (double) (village.func_180608_a().func_177956_o() + 4),
                        (double) (village.func_180608_a().func_177952_p() + village.func_75568_b())));

        // Count up the guards
        for (EntityVillagerMCA villager : list) {
            if (villager.getProfessionForge().getRegistryName().equals(ProfessionsMCA.guard.getRegistryName())) guards++;
        }

        // Spawn a new guard if we don't have enough, up to 10
        if (guards < guardCapacity && guards < 10) {
            Vec3d spawnPos = findRandomSpawnPos(world, village, village.func_180608_a(), 2, 4, 2);

            if (spawnPos != null) {
                EntityVillagerMCA guard = new EntityVillagerMCA(world, Optional.of(ProfessionsMCA.guard), Optional.absent());
                guard.func_70107_b(spawnPos.field_72450_a + 0.5D, spawnPos.field_72448_b + 1.0D, spawnPos.field_72449_c + 0.5D);
                guard.func_190672_a(world.func_175649_E(guard.func_190671_u_()), null, false);
                world.func_72838_d(guard);
            }
        }
    }

    private static void startRaid(World world, Village village) {
        int banditsToSpawn = world.field_73012_v.nextInt(5) + 1;

        while (banditsToSpawn > 0) {
            EntityVillagerMCA bandit = new EntityVillagerMCA(world, Optional.of(ProfessionsMCA.bandit), Optional.absent());
            BlockPos spawnLocation = village.func_180608_a();
            bandit.func_70107_b(spawnLocation.func_177958_n(), spawnLocation.func_177956_o(), spawnLocation.func_177952_p());
            world.func_72838_d(bandit);
            banditsToSpawn--;
        }
    }

    private static Vec3d findRandomSpawnPos(World world, Village village, BlockPos pos, int x, int y, int z) {
        for (int i = 0; i < 10; ++i) {
            BlockPos blockpos = pos.func_177982_a(world.field_73012_v.nextInt(16) - 8, world.field_73012_v.nextInt(6) - 3, world.field_73012_v.nextInt(16) - 8);

            if (village.func_179866_a(blockpos) && isAreaClearAround(world, new BlockPos(x, y, z), blockpos))
                return new Vec3d((double) blockpos.func_177958_n(), (double) blockpos.func_177956_o(), (double) blockpos.func_177952_p());
        }

        return null;
    }

    private static boolean isAreaClearAround(World world, BlockPos blockSize, BlockPos blockLocation) {
        if (!world.func_180495_p(blockLocation.func_177977_b()).func_185896_q()) return false;
        int i = blockLocation.func_177958_n() - blockSize.func_177958_n() / 2;
        int j = blockLocation.func_177952_p() - blockSize.func_177952_p() / 2;

        for (int k = i; k < i + blockSize.func_177958_n(); ++k) {
            for (int l = blockLocation.func_177956_o(); l < blockLocation.func_177956_o() + blockSize.func_177956_o(); ++l) {
                for (int i1 = j; i1 < j + blockSize.func_177952_p(); ++i1) {
                    if (world.func_180495_p(new BlockPos(k, l, i1)).func_185915_l()) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
}
