package mca.entity;


import mca.core.MCA;
import mca.core.minecraft.ItemsMCA;
import mca.core.minecraft.SoundsMCA;
import mca.enums.EnumReaperAttackState;
import mca.util.Util;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.effect.EntityLightningBolt;
import net.minecraft.entity.monster.EntityMob;
import net.minecraft.entity.monster.EntitySkeleton;
import net.minecraft.entity.monster.EntityZombie;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.init.Items;
import net.minecraft.init.MobEffects;
import net.minecraft.init.SoundEvents;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.ItemStack;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.potion.PotionEffect;
import net.minecraft.util.DamageSource;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.MathHelper;
import net.minecraft.world.BossInfo;
import net.minecraft.world.BossInfoServer;
import net.minecraft.world.World;

public class EntityGrimReaper extends EntityMob {
    private static final DataParameter<Integer> ATTACK_STATE = EntityDataManager.<Integer>func_187226_a(EntityGrimReaper.class, DataSerializers.field_187192_b);
    private static final DataParameter<Integer> STATE_TRANSITION_COOLDOWN = EntityDataManager.<Integer>func_187226_a(EntityGrimReaper.class, DataSerializers.field_187192_b);

    private final BossInfoServer bossInfo = (BossInfoServer) (new BossInfoServer(this.func_145748_c_(), BossInfo.Color.PURPLE, BossInfo.Overlay.PROGRESS)).func_186741_a(true);
    private EntityAINearestAttackableTarget aiNearestAttackableTarget = new EntityAINearestAttackableTarget(this, EntityPlayer.class, true);
    private int healingCooldown;
    private int timesHealed;

    private float floatingTicks;

    public EntityGrimReaper(World world) {
        super(world);
        func_70105_a(1.0F, 2.6F);
        this.field_70728_aV = 100;

        this.field_70714_bg.func_75776_a(1, new EntityAISwimming(this));
        this.field_70714_bg.func_75776_a(4, new EntityAIWander(this, 1.0D));
        this.field_70714_bg.func_75776_a(6, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.field_70714_bg.func_75776_a(6, new EntityAILookIdle(this));
        this.field_70715_bh.func_75776_a(1, new EntityAIHurtByTarget(this, false, new Class[0]));
        this.field_70715_bh.func_75776_a(2, aiNearestAttackableTarget);
    }

    @Override
    protected final void func_110147_ax() {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111265_b).func_111128_a(40.0D);
        this.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.30F);
        this.func_110148_a(SharedMonsterAttributes.field_111264_e).func_111128_a(12.5F);
        this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(225.0F);
    }

    @Override
    protected void func_70628_a(boolean hitByPlayer, int lootingLvl) {
        func_145779_a(ItemsMCA.STAFF_OF_LIFE, 1);
    }

    @Override
    protected void func_70088_a() {
        super.func_70088_a();
        this.field_70180_af.func_187214_a(ATTACK_STATE, 0);
        this.field_70180_af.func_187214_a(STATE_TRANSITION_COOLDOWN, 0);
    }

    public EnumReaperAttackState getAttackState() {
        return EnumReaperAttackState.fromId(this.field_70180_af.func_187225_a(ATTACK_STATE));
    }

    public void setAttackState(EnumReaperAttackState state) {
        // Only update if needed so that sounds only play once.
        if (this.field_70180_af.func_187225_a(ATTACK_STATE) != state.getId()) {
            this.field_70180_af.func_187227_b(ATTACK_STATE, state.getId());

            switch (state) {
                case PRE:
                    this.func_184185_a(SoundsMCA.reaper_scythe_out, 1.0F, 1.0F);
                    break;
                case POST:
                    this.func_184185_a(SoundsMCA.reaper_scythe_swing, 1.0F, 1.0F);
                    break;
            }
        }
    }

    public boolean hasEntityToAttack() {
        return this.func_70638_az() != null;
    }

    @Override
    public void func_70077_a(EntityLightningBolt entity) {
        return;
    }

    @Override
    public boolean func_70097_a(DamageSource source, float damage) {
        bossInfo.func_186735_a(this.func_110143_aJ() / this.func_110138_aP());

        // Ignore wall damage and fire damage.
        if (source == DamageSource.field_76368_d || source == DamageSource.field_76370_b || source.func_94541_c() || source == DamageSource.field_76372_a) {
            // Teleport out of any walls we may end up in.
            if (source == DamageSource.field_76368_d) {
                teleportTo(this.field_70165_t, this.field_70163_u + 3, this.field_70161_v);
            }

            return false;
        }

        // Ignore damage when blocking, and teleport behind the player when they attempt to block.
        else if (!field_70170_p.field_72995_K && this.getAttackState() == EnumReaperAttackState.BLOCK && source.func_76364_f() instanceof EntityPlayer) {
            EntityPlayer player = (EntityPlayer) source.func_76364_f();

            double deltaX = this.field_70165_t - player.field_70165_t;
            double deltaZ = this.field_70161_v - player.field_70161_v;

            this.func_184185_a(SoundsMCA.reaper_block, 1.0F, 1.0F);
            teleportTo(player.field_70165_t - (deltaX * 2), player.field_70163_u + 2, this.field_70161_v - (deltaZ * 2));
            setStateTransitionCooldown(0);
            return false;
        }

        // Randomly portal behind the player who just attacked.
        else if (!field_70170_p.field_72995_K && source.func_76364_f() instanceof EntityPlayer && field_70146_Z.nextFloat() >= 0.30F) {
            EntityPlayer player = (EntityPlayer) source.func_76364_f();

            double deltaX = this.field_70165_t - player.field_70165_t;
            double deltaZ = this.field_70161_v - player.field_70161_v;

            teleportTo(player.field_70165_t - (deltaX * 2), player.field_70163_u + 2, this.field_70161_v - (deltaZ * 2));
        }

        // Teleport behind the player who fired an arrow and ignore its damage.
        else if (source.func_76364_f() instanceof EntityArrow) {
            EntityArrow arrow = (EntityArrow) source.func_76364_f();

            if (arrow.field_70250_c instanceof EntityPlayer && getAttackState() != EnumReaperAttackState.REST) {
                EntityPlayer player = (EntityPlayer) arrow.field_70250_c;
                double newX = player.field_70165_t + field_70146_Z.nextFloat() >= 0.50F ? 2 : -2;
                double newZ = player.field_70161_v + field_70146_Z.nextFloat() >= 0.50F ? 2 : -2;

                teleportTo(newX, player.field_70163_u, newZ);
            }

            arrow.func_70106_y();
            return false;
        }

        // Still take damage when healing, but reduced by a third.
        else if (this.getAttackState() == EnumReaperAttackState.REST) {
            damage /= 3;
        }

        super.func_70097_a(source, damage);

        if (!field_70170_p.field_72995_K && this.func_110143_aJ() <= (this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111125_b() / 2) && healingCooldown == 0) {
            setAttackState(EnumReaperAttackState.REST);
            healingCooldown = 4200; // 3 minutes 30 seconds
            teleportTo(this.field_70165_t, this.field_70163_u + 8, this.field_70161_v);
            setStateTransitionCooldown(1200); // 1 minute
        }

        return true;
    }

    protected void attackEntity(Entity entity, float damage) {
        EntityLivingBase entityToAttack = this.func_70638_az();
        if (entityToAttack == null) return;

        // Set attack state to post attack.
        // If we're blocking, we will teleport away instead of attacking to prevent an unfair attack.
        // Attacking us WHILE we're blocking will cause us to attack, however.
        if (this.func_70032_d(entityToAttack) <= 0.8D && getAttackState() == EnumReaperAttackState.PRE) {
            if (getAttackState() == EnumReaperAttackState.BLOCK) {
                int rX = this.func_70681_au().nextInt(10);
                int rZ = this.func_70681_au().nextInt(10);
                teleportTo(this.field_70165_t + 5 + rX, this.field_70163_u, this.field_70161_v + rZ);
            } else {
                entity.func_70097_a(DamageSource.func_76358_a(this), this.field_70170_p.func_175659_aa().func_151525_a() * 5.75F);

                if (entity instanceof EntityLivingBase) {
                    ((EntityLivingBase) entity).func_70690_d(new PotionEffect(MobEffects.field_82731_v, this.field_70170_p.func_175659_aa().func_151525_a() * 20, 1));
                }

                setAttackState(EnumReaperAttackState.POST);
                setStateTransitionCooldown(10); // For preventing immediate return to the PRE or IDLE stage. Ticked down in onUpdate()
            }
        }

        // Check if we're waiting for cooldown from the last attack.
        if (getStateTransitionCooldown() == 0) {
            // Within 3 blocks from the target, ready the scythe
            if (func_70032_d(entityToAttack) <= 3.5D) {
                // Check to see if the player's blocking, then teleport behind them.
                // Also randomly swap their selected item with something else in the hotbar and apply blindness.
                if (entityToAttack instanceof EntityPlayer) {
                    EntityPlayer player = (EntityPlayer) entityToAttack;

                    if (player.func_184585_cz()) {
                        double dX = this.field_70165_t - player.field_70165_t;
                        double dZ = this.field_70161_v - player.field_70161_v;

                        teleportTo(player.field_70165_t - (dX * 2), player.field_70163_u + 2, this.field_70161_v - (dZ * 2));

                        if (!field_70170_p.field_72995_K && field_70146_Z.nextFloat() >= 0.20F) {
                            int currentItem = player.field_71071_by.field_70461_c;
                            int randomItem = field_70146_Z.nextInt(InventoryPlayer.func_70451_h());
                            ItemStack currentItemStack = player.field_71071_by.field_70462_a.get(currentItem);
                            ItemStack randomItemStack = player.field_71071_by.field_70462_a.get(randomItem);

                            player.field_71071_by.field_70462_a.set(currentItem, randomItemStack);
                            player.field_71071_by.field_70462_a.set(randomItem, currentItemStack);

                            player.func_70690_d(new PotionEffect(MobEffects.field_76440_q, this.field_70170_p.func_175659_aa().func_151525_a() * 40, 1));
                        }
                    } else // If the player is not blocking, ready the scythe, or randomly block their attack.
                    {
                        // Don't block if we've already committed to an attack.
                        if (field_70146_Z.nextFloat() >= 40.0F && getAttackState() != EnumReaperAttackState.PRE) {
                            setStateTransitionCooldown(20);
                            setAttackState(EnumReaperAttackState.BLOCK);
                        } else {
                            setAttackState(EnumReaperAttackState.PRE);
                            setStateTransitionCooldown(20);
                        }
                    }
                }
            } else // Reset the attacking state when we're more than 3 blocks away.
            {
                setAttackState(EnumReaperAttackState.IDLE);
            }
        }
    }

    protected Entity findPlayerToAttack() {
        return field_70170_p.func_72890_a(this, 48.0D);
    }

    @Override
    public int func_70627_aG() {
        return 300;
    }

    @Override
    protected SoundEvent func_184639_G() {
        return SoundsMCA.reaper_idle;
    }

    @Override
    protected SoundEvent func_184615_bR() {
        return SoundsMCA.reaper_death;
    }

    @Override
    protected SoundEvent func_184601_bQ(DamageSource source) {
        return SoundEvents.field_187851_gB;
    }

    @Override
    public void func_70071_h_() {
        super.func_70071_h_();
        func_70066_B(); // No fire.

        if (!MCA.getConfig().allowGrimReaper) {
            func_70106_y();
        }

        EntityLivingBase entityToAttack = this.func_70638_az();

        if (entityToAttack != null && getAttackState() != EnumReaperAttackState.REST) {
            attackEntity(entityToAttack, 5.0F);
            this.func_70605_aq().func_75642_a(entityToAttack.field_70165_t, entityToAttack.field_70163_u, entityToAttack.field_70161_v, 6.0F);
        }

        // Increment floating ticks on the client when resting.
        if (field_70170_p.field_72995_K && getAttackState() == EnumReaperAttackState.REST) {
            floatingTicks += 0.1F;
        }

        // Increase health when resting and check to stop rest state.
        // Runs on common to spawn lightning.
        if (getAttackState() == EnumReaperAttackState.REST) {
            if (!field_70170_p.field_72995_K && getStateTransitionCooldown() == 1) {
                setAttackState(EnumReaperAttackState.IDLE);
                timesHealed++;
            } else if (!field_70170_p.field_72995_K && getStateTransitionCooldown() % 100 == 0) {
                this.func_70606_j(this.func_110143_aJ() + MathHelper.func_76131_a(10.5F - (timesHealed * 3.5F), 3.0F, 10.5F));

                // Let's have a light show.
                int dX = field_70146_Z.nextInt(8) + 4 * field_70146_Z.nextFloat() >= 0.50F ? 1 : -1;
                int dZ = field_70146_Z.nextInt(8) + 4 * field_70146_Z.nextFloat() >= 0.50F ? 1 : -1;
                int y = Util.getSpawnSafeTopLevel(field_70170_p, (int) field_70165_t + dX, 256, (int) field_70161_v + dZ);

                EntityLightningBolt bolt = new EntityLightningBolt(field_70170_p, dX, y, dZ, false);
                field_70170_p.func_72942_c(bolt);

                // Also spawn a random skeleton or zombie.
                if (!field_70170_p.field_72995_K) {
                    EntityMob mob = field_70146_Z.nextFloat() >= 0.50F ? new EntityZombie(field_70170_p) : new EntitySkeleton(field_70170_p);
                    mob.func_70107_b(field_70165_t + dX + 4, y, field_70161_v + dZ + 4);

                    if (mob instanceof EntitySkeleton) {
                        mob.func_184201_a(EntityEquipmentSlot.MAINHAND, new ItemStack(Items.field_151031_f));
                    }

                    field_70170_p.func_72838_d(mob);
                }
            }
        }

        // Prevent flying off into oblivion on death...
        if (this.func_110143_aJ() <= 0.0F) {
            field_70159_w = 0;
            field_70181_x = 0;
            field_70179_y = 0;
            return;
        }

        // Stop at our current position if resting
        if (getAttackState() == EnumReaperAttackState.REST) {
            field_70159_w = 0;
            field_70181_x = 0;
            field_70179_y = 0;
        }

        // Logic for flying.
        field_70143_R = 0.0F;

        if (field_70181_x > 0) {
            field_70181_x = field_70181_x * 1.04F;
        } else {
            double yMod = Math.sqrt((field_70159_w * field_70159_w) + (field_70179_y * field_70179_y));
            field_70181_x = field_70181_x * 0.6F + yMod * 0.3F;
        }

        // Tick down cooldowns.
        if (getStateTransitionCooldown() > 0) {
            setStateTransitionCooldown(getStateTransitionCooldown() - 1);
        }

        if (healingCooldown > 0) {
            healingCooldown--;
        }

        // See if our entity to attack has died at any point.
        if (entityToAttack != null && entityToAttack.field_70128_L) {
            this.func_70624_b(null);
            setAttackState(EnumReaperAttackState.IDLE);
        }

        // Move towards target if we're not resting
        if (entityToAttack != null && getAttackState() != EnumReaperAttackState.REST) {
            // If we have a creature to attack, we need to move downwards if we're above it, and vice-versa.
            double sqDistanceTo = Math.sqrt(Math.pow(entityToAttack.field_70165_t - field_70165_t, 2) + Math.pow(entityToAttack.field_70161_v - field_70161_v, 2));
            float moveAmount = 0.0F;

            if (sqDistanceTo < 8F) {
                moveAmount = MathHelper.func_76131_a(((8F - (float) sqDistanceTo) / 8F) * 4F, 0, 2.5F);
            }

            if (entityToAttack.field_70163_u + 0.2F < field_70163_u) {
                field_70181_x = field_70181_x - 0.05F * moveAmount;
            }

            if (entityToAttack.field_70163_u - 0.5F > field_70163_u) {
                field_70181_x = field_70181_x + 0.01F * moveAmount;
            }
        }
    }

    @Override
    public void func_70645_a(DamageSource source) {
        super.func_70645_a(source);
    }

    @Override
    public String func_70005_c_() {
        return "Grim Reaper";
    }

    @Override
    protected boolean func_70692_ba() {
        return true;
    }

    public int getStateTransitionCooldown() {
        return this.field_70180_af.func_187225_a(STATE_TRANSITION_COOLDOWN);
    }

    public void setStateTransitionCooldown(int value) {
        this.field_70180_af.func_187227_b(STATE_TRANSITION_COOLDOWN, value);
    }

    public float getFloatingTicks() {
        return floatingTicks;
    }

    private void teleportTo(double x, double y, double z) {
        if (!field_70170_p.field_72995_K) {
            this.func_184185_a(SoundEvents.field_187534_aX, 2.0F, 1.0F);
            this.func_70107_b(x, y, z);
            this.func_184185_a(SoundEvents.field_187534_aX, 2.0F, 1.0F);
        }
    }

    @Override
    public boolean func_184222_aU() {
        return false;
    }

    /**
     * Add the given player to the list of players tracking this entity. For instance, a player may track a boss in
     * order to view its associated boss bar.
     */
    @Override
    public void func_184178_b(EntityPlayerMP player) {
        super.func_184178_b(player);
        this.bossInfo.func_186760_a(player);
    }

    /**
     * Removes the given player from the list of players tracking this entity. See {@link Entity#addTrackingPlayer} for
     * more information on tracking.
     */
    @Override
    public void func_184203_c(EntityPlayerMP player) {
        super.func_184203_c(player);
        this.bossInfo.func_186761_b(player);
    }

}
