package mca.entity;

import com.google.common.base.Optional;
import com.google.common.base.Predicate;
import mca.api.API;
import mca.api.types.APIButton;
import mca.core.Constants;
import mca.core.MCA;
import mca.core.forge.NetMCA;
import mca.core.minecraft.ItemsMCA;
import mca.core.minecraft.ProfessionsMCA;
import mca.entity.ai.*;
import mca.entity.data.ParentData;
import mca.entity.data.PlayerHistory;
import mca.entity.data.PlayerSaveData;
import mca.entity.data.SavedVillagers;
import mca.entity.inventory.InventoryMCA;
import mca.enums.*;
import mca.items.ItemSpecialCaseGift;
import mca.util.ItemStackCache;
import mca.util.ResourceLocationCache;
import mca.util.Util;
import net.minecraft.block.BlockBed;
import net.minecraft.block.BlockHorizontal;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.*;
import net.minecraft.entity.monster.EntityVex;
import net.minecraft.entity.monster.EntityVindicator;
import net.minecraft.entity.monster.EntityZombie;
import net.minecraft.entity.passive.EntityHorse;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.DataSerializers;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.util.*;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.util.registry.RegistryNamespaced;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraft.world.World;
import net.minecraftforge.fml.common.ObfuscationReflectionHelper;
import net.minecraftforge.fml.common.registry.VillagerRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.UUID;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;


import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;

import static net.minecraft.block.BlockBed.OCCUPIED;
import static net.minecraft.block.BlockBed.PART;

public class EntityVillagerMCA extends EntityVillager {
    public static final int VANILLA_CAREER_ID_FIELD_INDEX = 13;
    public static final int VANILLA_CAREER_LEVEL_FIELD_INDEX = 14;

    public static final DataParameter<String> VILLAGER_NAME = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187194_d);
    public static final DataParameter<String> TEXTURE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187194_d);
    public static final DataParameter<Integer> GENDER = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187192_b);
    public static final DataParameter<Float> GIRTH = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187193_c);
    public static final DataParameter<Float> TALLNESS = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187193_c);
    public static final DataParameter<NBTTagCompound> PLAYER_HISTORY_MAP = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_192734_n);
    public static final DataParameter<Integer> MOVE_STATE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187192_b);
    public static final DataParameter<String> SPOUSE_NAME = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187194_d);
    public static final DataParameter<Optional<UUID>> SPOUSE_UUID = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187203_m);
    public static final DataParameter<Integer> MARRIAGE_STATE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187192_b);
    public static final DataParameter<Boolean> IS_PROCREATING = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187198_h);
    public static final DataParameter<NBTTagCompound> PARENTS = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_192734_n);
    public static final DataParameter<Boolean> IS_INFECTED = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187198_h);
    public static final DataParameter<Integer> AGE_STATE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187192_b);
    public static final DataParameter<Integer> ACTIVE_CHORE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187192_b);
    public static final DataParameter<Boolean> IS_SWINGING = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187198_h);
    public static final DataParameter<Boolean> HAS_BABY = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187198_h);
    public static final DataParameter<Boolean> BABY_IS_MALE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187198_h);
    public static final DataParameter<Integer> BABY_AGE = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187192_b);
    public static final DataParameter<Optional<UUID>> CHORE_ASSIGNING_PLAYER = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187203_m);
    public static final DataParameter<BlockPos> BED_POS = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187200_j);
    public static final DataParameter<BlockPos> WORKPLACE_POS = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187200_j);
    public static final DataParameter<BlockPos> HANGOUT_POS = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187200_j);
    public static final DataParameter<Boolean> SLEEPING = EntityDataManager.func_187226_a(EntityVillagerMCA.class, DataSerializers.field_187198_h);

    private static final Predicate<EntityVillagerMCA> BANDIT_TARGET_SELECTOR = (v) -> v.getProfessionForge() != ProfessionsMCA.bandit && v.getProfessionForge() != ProfessionsMCA.child;
    private static final Predicate<EntityVillagerMCA> GUARD_TARGET_SELECTOR = (v) -> v.getProfessionForge() == ProfessionsMCA.bandit;

    public final InventoryMCA inventory;
    public int babyAge = 0;
    public UUID playerToFollowUUID = Constants.ZERO_UUID;

    private BlockPos home = BlockPos.field_177992_a;
    private int startingAge = 0;
    private float swingProgressTicks;

    public float renderOffsetX;
    public float renderOffsetY;
    public float renderOffsetZ;

    public EntityVillagerMCA() {
        super(null);
        inventory = null;
    }

    public EntityVillagerMCA(World worldIn) {
        super(worldIn);
        inventory = new InventoryMCA(this);
    }

    public EntityVillagerMCA(World worldIn, Optional<VillagerRegistry.VillagerProfession> profession, Optional<EnumGender> gender) {
        this(worldIn);

        if (!worldIn.field_72995_K) {
            EnumGender eGender = gender.isPresent() ? gender.get() : EnumGender.getRandom();
            set(GENDER, eGender.getId());
            set(VILLAGER_NAME, API.getRandomName(eGender));
            setProfession(profession.isPresent() ? profession.get() : ProfessionsMCA.randomProfession());
            setVanillaCareer(getProfessionForge().getRandomCareer(worldIn.field_73012_v));
            set(TEXTURE, API.getRandomSkin(this));

            applySpecialAI();
        }
    }

    @Override
    protected void func_70088_a() {
        super.func_70088_a();
        this.field_70180_af.func_187214_a(VILLAGER_NAME, "");
        this.field_70180_af.func_187214_a(TEXTURE, "");
        this.field_70180_af.func_187214_a(GENDER, EnumGender.MALE.getId());
        this.field_70180_af.func_187214_a(GIRTH, 0.0F);
        this.field_70180_af.func_187214_a(TALLNESS, 0.0F);
        this.field_70180_af.func_187214_a(PLAYER_HISTORY_MAP, new NBTTagCompound());
        this.field_70180_af.func_187214_a(MOVE_STATE, EnumMoveState.MOVE.getId());
        this.field_70180_af.func_187214_a(SPOUSE_NAME, "");
        this.field_70180_af.func_187214_a(SPOUSE_UUID, Optional.of(Constants.ZERO_UUID));
        this.field_70180_af.func_187214_a(MARRIAGE_STATE, EnumMarriageState.NOT_MARRIED.getId());
        this.field_70180_af.func_187214_a(IS_PROCREATING, false);
        this.field_70180_af.func_187214_a(PARENTS, new NBTTagCompound());
        this.field_70180_af.func_187214_a(IS_INFECTED, false);
        this.field_70180_af.func_187214_a(AGE_STATE, EnumAgeState.ADULT.getId());
        this.field_70180_af.func_187214_a(ACTIVE_CHORE, EnumChore.NONE.getId());
        this.field_70180_af.func_187214_a(IS_SWINGING, false);
        this.field_70180_af.func_187214_a(HAS_BABY, false);
        this.field_70180_af.func_187214_a(BABY_IS_MALE, false);
        this.field_70180_af.func_187214_a(BABY_AGE, 0);
        this.field_70180_af.func_187214_a(CHORE_ASSIGNING_PLAYER, Optional.of(Constants.ZERO_UUID));
        this.field_70180_af.func_187214_a(BED_POS, BlockPos.field_177992_a);
        this.field_70180_af.func_187214_a(WORKPLACE_POS, BlockPos.field_177992_a);
        this.field_70180_af.func_187214_a(HANGOUT_POS, BlockPos.field_177992_a);
        this.field_70180_af.func_187214_a(SLEEPING, false);
        this.func_174810_b(false);
    }

    @Override
    protected void func_110147_ax() {
        super.func_110147_ax();
        this.func_110148_a(SharedMonsterAttributes.field_111267_a).func_111128_a(MCA.getConfig().villagerMaxHealth);
        this.func_110148_a(SharedMonsterAttributes.field_111265_b).func_111128_a(32.0D);

        if (this.func_110143_aJ() <= MCA.getConfig().villagerMaxHealth) {
            this.func_70606_j(MCA.getConfig().villagerMaxHealth);
        }
    }

    public <T> T get(DataParameter<T> key) {
        return this.field_70180_af.func_187225_a(key);
    }

    public <T> void set(DataParameter<T> key, T value) {
        this.field_70180_af.func_187227_b(key, value);
    }

    @Override
    public boolean func_70652_k(@Nonnull Entity entityIn) {
        super.func_70652_k(entityIn);
        return entityIn.func_70097_a(DamageSource.func_76358_a(this), this.getProfessionForge() == ProfessionsMCA.guard ? 9.0F : 2.0F);
    }

    @Override
    public void func_70037_a(NBTTagCompound nbt) {
        super.func_70037_a(nbt);
        set(VILLAGER_NAME, nbt.func_74779_i("name"));
        set(GENDER, nbt.func_74762_e("gender"));
        set(TEXTURE, nbt.func_74779_i("texture"));
        set(GIRTH, nbt.func_74760_g("girth"));
        set(TALLNESS, nbt.func_74760_g("tallness"));
        set(PLAYER_HISTORY_MAP, nbt.func_74775_l("playerHistoryMap"));
        set(MOVE_STATE, nbt.func_74762_e("moveState"));
        set(MARRIAGE_STATE, nbt.func_74762_e("marriageState"));
        set(SPOUSE_UUID, Optional.of(nbt.func_186857_a("spouseUUID")));
        set(SPOUSE_NAME, nbt.func_74779_i("spouseName"));
        set(IS_PROCREATING, nbt.func_74767_n("isProcreating"));
        set(IS_INFECTED, nbt.func_74767_n("infected"));
        set(AGE_STATE, nbt.func_74762_e("ageState"));
        set(ACTIVE_CHORE, nbt.func_74762_e("activeChore"));
        set(CHORE_ASSIGNING_PLAYER, Optional.of(nbt.func_186857_a("choreAssigningPlayer")));
        set(HAS_BABY, nbt.func_74767_n("hasBaby"));
        set(BABY_IS_MALE, nbt.func_74767_n("babyIsMale"));
        set(PARENTS, nbt.func_74775_l("parents"));
        set(BED_POS, new BlockPos(nbt.func_74762_e("bedX"), nbt.func_74762_e("bedY"), nbt.func_74762_e("bedZ")));
        set(HANGOUT_POS, new BlockPos(nbt.func_74762_e("hangoutX"), nbt.func_74762_e("hangoutY"), nbt.func_74762_e("hangoutZ")));
        set(WORKPLACE_POS, new BlockPos(nbt.func_74762_e("workplaceX"), nbt.func_74762_e("workplaceY"), nbt.func_74762_e("workplaceZ")));
        set(SLEEPING, nbt.func_74767_n("sleeping"));
        inventory.readInventoryFromNBT(nbt.func_150295_c("inventory", 10));

        // Vanilla Age doesn't apply from the superclass call. Causes children to revert to the starting age on world reload.
        this.startingAge = nbt.func_74762_e("startingAge");
        func_70873_a(nbt.func_74762_e("Age"));

        this.home = new BlockPos(nbt.func_74769_h("homePositionX"), nbt.func_74769_h("homePositionY"), nbt.func_74769_h("homePositionZ"));
        this.playerToFollowUUID = nbt.func_186857_a("playerToFollowUUID");
        this.babyAge = nbt.func_74762_e("babyAge");

        applySpecialAI();
    }

    @Override
    public void func_70014_b(NBTTagCompound nbt) {
        super.func_70014_b(nbt);
        nbt.func_186854_a("uuid", this.func_110124_au()); // for SavedVillagers
        nbt.func_74778_a("name", get(VILLAGER_NAME));
        nbt.func_74778_a("texture", get(TEXTURE));
        nbt.func_74768_a("gender", get(GENDER));
        nbt.func_74776_a("girth", get(GIRTH));
        nbt.func_74776_a("tallness", get(TALLNESS));
        nbt.func_74782_a("playerHistoryMap", get(PLAYER_HISTORY_MAP));
        nbt.func_74768_a("moveState", get(MOVE_STATE));
        nbt.func_74768_a("marriageState", get(MARRIAGE_STATE));
        nbt.func_74780_a("homePositionX", home.func_177958_n());
        nbt.func_74780_a("homePositionY", home.func_177956_o());
        nbt.func_74780_a("homePositionZ", home.func_177952_p());
        nbt.func_186854_a("playerToFollowUUID", playerToFollowUUID);
        nbt.func_186854_a("spouseUUID", get(SPOUSE_UUID).or(Constants.ZERO_UUID));
        nbt.func_74778_a("spouseName", get(SPOUSE_NAME));
        nbt.func_74757_a("isProcreating", get(IS_PROCREATING));
        nbt.func_74757_a("infected", get(IS_INFECTED));
        nbt.func_74768_a("ageState", get(AGE_STATE));
        nbt.func_74768_a("startingAge", startingAge);
        nbt.func_74768_a("activeChore", get(ACTIVE_CHORE));
        nbt.func_186854_a("choreAssigningPlayer", get(CHORE_ASSIGNING_PLAYER).or(Constants.ZERO_UUID));
        nbt.func_74782_a("inventory", inventory.writeInventoryToNBT());
        nbt.func_74768_a("babyAge", babyAge);
        nbt.func_74782_a("parents", get(PARENTS));
        nbt.func_74768_a("bedX", get(BED_POS).func_177958_n());
        nbt.func_74768_a("bedY", get(BED_POS).func_177956_o());
        nbt.func_74768_a("bedZ", get(BED_POS).func_177952_p());
        nbt.func_74768_a("workplaceX", get(WORKPLACE_POS).func_177958_n());
        nbt.func_74768_a("workplaceY", get(WORKPLACE_POS).func_177956_o());
        nbt.func_74768_a("workplaceZ", get(WORKPLACE_POS).func_177952_p());
        nbt.func_74768_a("hangoutX", get(HANGOUT_POS).func_177958_n());
        nbt.func_74768_a("hangoutY", get(HANGOUT_POS).func_177956_o());
        nbt.func_74768_a("hangoutZ", get(HANGOUT_POS).func_177952_p());
        nbt.func_74757_a("sleeping", get(SLEEPING));
    }

    @Override
    protected void func_70665_d(@Nonnull DamageSource damageSource, float damageAmount) {
        // Guards take 50% less damage
        if (getProfessionForge() == ProfessionsMCA.guard) {
            damageAmount *= 0.50;
        }
        super.func_70665_d(damageSource, damageAmount);

        // Check for infection to apply. Does not affect guards.
        if (MCA.getConfig().enableInfection && getProfessionForge() != ProfessionsMCA.guard && damageSource.func_76364_f() instanceof EntityZombie && func_70681_au().nextFloat() < MCA.getConfig().infectionChance / 100) {
            set(IS_INFECTED, true);
        }
    }

    @Override
    public void func_70071_h_() {
        super.func_70071_h_();
        updateSwinging();
        updateSleeping();

        if (this.func_70613_aW()) {
            onEachServerUpdate();
        } else {
            onEachClientUpdate();
        }
    }

    @Override
    protected SoundEvent func_184639_G() {
        return null;
    }

    @Override
    protected SoundEvent func_184601_bQ(DamageSource damageSourceIn) {
        return SoundEvents.field_187543_bD;
    }

    @Override
    protected SoundEvent func_184615_bR() {
        return get(IS_INFECTED) ? SoundEvents.field_187930_hd : null;
    }

    @Override
    public boolean func_184645_a(EntityPlayer player, @Nonnull EnumHand hand) {
        // No-op, handled by EventHooks
        return true;
    }

    @Override
    public void func_70645_a(@Nonnull DamageSource cause) {
        if (!field_70170_p.field_72995_K) {
            String causeName = cause.func_76364_f() == null ? cause.func_76355_l() : cause.func_76364_f().func_70005_c_();
            if (MCA.getConfig().logVillagerDeaths) {
                MCA.getLog().info("Villager death: " + get(VILLAGER_NAME) + ". Caused by: " + causeName + ". UUID: " + this.func_110124_au().toString());
            }

            inventory.dropAllItems();

            if (isMarried()) {
                UUID spouseUUID = get(SPOUSE_UUID).or(Constants.ZERO_UUID);
                Optional<EntityVillagerMCA> spouse = Util.getEntityByUUID(field_70170_p, spouseUUID, EntityVillagerMCA.class);
                PlayerSaveData playerSaveData = PlayerSaveData.getExisting(field_70170_p, spouseUUID);

                // Notify spouse of the death
                if (spouse.isPresent()) {
                    spouse.get().endMarriage();
                } else if (playerSaveData != null) {
                    playerSaveData.endMarriage();
                    EntityPlayer player = field_70170_p.func_152378_a(spouseUUID);
                    if (player != null) {
                        player.func_145747_a(new TextComponentString(Constants.Color.RED + MCA.getLocalizer().localize("notify.spousedied", get(VILLAGER_NAME), causeName)));
                    }
                }
            }

            // Notify all parents of the death
            ParentData parents = ParentData.fromNBT(get(PARENTS));
            Arrays.stream(parents.getParentEntities(field_70170_p))
                    .filter(e -> e instanceof EntityPlayer)
                    .forEach(e -> {
                        EntityPlayer player = (EntityPlayer) e;
                        player.func_145747_a(new TextComponentString(Constants.Color.RED + MCA.getLocalizer().localize("notify.childdied", get(VILLAGER_NAME), causeName)));
                    });

            SavedVillagers.get(field_70170_p).save(this);
        }
    }

    @Override
    protected void func_175500_n() {
        Entity[] parents = ParentData.fromNBT(get(PARENTS)).getParentEntities(field_70170_p);
        set(AGE_STATE, EnumAgeState.ADULT.getId());
        Arrays.stream(parents).filter((e) -> e instanceof EntityPlayer).forEach((e) -> {
            PlayerHistory history = getPlayerHistoryFor(e.func_110124_au());
            history.setDialogueType(EnumDialogueType.ADULT);
            e.func_145747_a(new TextComponentString(MCA.getLocalizer().localize("notify.child.grownup", this.get(VILLAGER_NAME))));
        });

        // set profession away from child for villager children
        if (getProfessionForge() == ProfessionsMCA.child) {
            setProfession(ProfessionsMCA.randomProfession());
            setVanillaCareer(getProfessionForge().getRandomCareer(field_70170_p.field_73012_v));
        }
    }

    @Override
    @Nonnull
    public ITextComponent func_145748_c_() {
        // translate profession name
        ITextComponent careerName = new TextComponentTranslation("entity.Villager." + getVanillaCareer().getName());
        EnumAgeState age = EnumAgeState.byId(get(AGE_STATE));
        String professionName = age != EnumAgeState.ADULT ? age.localizedName() : careerName.func_150260_c();
        String color = this.getProfessionForge() == ProfessionsMCA.bandit ? Constants.Color.RED : this.getProfessionForge() == ProfessionsMCA.guard ? Constants.Color.GREEN : "";

        return new TextComponentString(String.format("%1$s%2$s%3$s (%4$s)", color, MCA.getConfig().villagerChatPrefix, get(VILLAGER_NAME), professionName));
    }

    @Override
    @Nonnull
    public String func_95999_t() {
        return get(VILLAGER_NAME);
    }

    @Override
    @Nonnull
    public boolean func_145818_k_() {
        return true;
    }

    @Override
    public void func_184609_a(EnumHand hand) {
        this.func_184598_c(EnumHand.MAIN_HAND);
        super.func_184609_a(EnumHand.MAIN_HAND);

        if (!get(IS_SWINGING) || swingProgressTicks >= 8 / 2 || swingProgressTicks < 0) {
            swingProgressTicks = -1;
            set(IS_SWINGING, true);
        }
    }

    private void updateSwinging() {
        if (get(IS_SWINGING)) {
            swingProgressTicks++;

            if (swingProgressTicks >= 8) {
                swingProgressTicks = 0;
                set(IS_SWINGING, false);
            }
        } else {
            swingProgressTicks = 0;
        }
        field_70733_aJ = swingProgressTicks / (float) 8;
    }

    @Override
    @Nonnull
    public ItemStack func_184582_a(EntityEquipmentSlot slotIn) {
        if (slotIn == EntityEquipmentSlot.MAINHAND) {
            VillagerRegistry.VillagerProfession profession = getProfessionForge();
            EnumChore chore = EnumChore.byId(get(ACTIVE_CHORE));
            if (get(HAS_BABY)) {
                return ItemStackCache.get(get(BABY_IS_MALE) ? ItemsMCA.BABY_BOY : ItemsMCA.BABY_GIRL);
            } else if (chore != EnumChore.NONE) {
                return inventory.getBestItemOfType(chore.getToolType());
            } else {
                return ProfessionsMCA.getDefaultHeldItem(profession, getVanillaCareer());
            }
        } else {
            return inventory.getBestArmorOfType(slotIn);
        }
    }

    public void setStartingAge(int value) {
        this.startingAge = value;
        func_70873_a(value);
    }

    public PlayerHistory getPlayerHistoryFor(UUID uuid) {
        if (!get(PLAYER_HISTORY_MAP).func_74764_b(uuid.toString())) {
            updatePlayerHistoryMap(PlayerHistory.getNew(this, uuid));
        }
        return PlayerHistory.fromNBT(this, uuid, get(PLAYER_HISTORY_MAP).func_74775_l(uuid.toString()));
    }

    public void updatePlayerHistoryMap(PlayerHistory history) {
        NBTTagCompound nbt = get(PLAYER_HISTORY_MAP);
        nbt.func_74782_a(history.getPlayerUUID().toString(), history.toNBT());
        set(PLAYER_HISTORY_MAP, nbt);
        this.field_70180_af.func_187217_b(PLAYER_HISTORY_MAP);
    }

    public void reset() {
        set(PLAYER_HISTORY_MAP, new NBTTagCompound());
        field_70180_af.func_187217_b(PLAYER_HISTORY_MAP);

        func_70606_j(20.0F);

        set(SPOUSE_NAME, "");
        set(SPOUSE_UUID, Optional.of(Constants.ZERO_UUID));
        set(MARRIAGE_STATE, EnumMarriageState.NOT_MARRIED.getId());
        set(HAS_BABY, false);
    }

    public VillagerRegistry.VillagerCareer getVanillaCareer() {
        return this.getProfessionForge().getCareer(ObfuscationReflectionHelper.getPrivateValue(EntityVillager.class, this, VANILLA_CAREER_ID_FIELD_INDEX));
    }

    public void setVanillaCareer(int careerId) {
        ObfuscationReflectionHelper.setPrivateValue(EntityVillager.class, this, careerId, VANILLA_CAREER_ID_FIELD_INDEX);
    }

    private void setSizeForAge() {
        EnumAgeState age = EnumAgeState.byId(get(AGE_STATE));
        this.func_70105_a(age.getWidth(), age.getHeight());
        this.func_98055_j(1.0F); // trigger rebuild of the bounding box
    }

    private void toggleMount(EntityPlayerMP player) {
        if (func_184187_bx() != null) {
            func_184210_p();
        } else {
            try {
                List<EntityHorse> horses = field_70170_p.func_175644_a(EntityHorse.class, h -> (h.func_110257_ck() && !h.func_184207_aI() && h.func_70032_d(this) < 3.0D));
                func_184205_a(horses.stream().min(Comparator.comparingDouble(this::func_70032_d)).get(), true);
                func_70661_as().func_75499_g();
            } catch (NoSuchElementException e) {
                say(Optional.of(player), "interaction.ridehorse.fail.notnearby");
            }
        }
    }

    private void goHome(EntityPlayerMP player) {
        if (home.equals(Vec3d.field_186680_a)) {
            say(Optional.of(player), "interaction.gohome.fail");
        } else {
            say(Optional.of(player), "interaction.gohome.success");
            if (!func_70661_as().func_75484_a(func_70661_as().func_75488_a(home.func_177958_n(), home.func_177956_o(), home.func_177952_p()), 1.0D)) {
                func_184595_k(home.func_177958_n(), home.func_177956_o(), home.func_177952_p());
            }
        }
    }

    public BlockPos getWorkplace() {
        return get(WORKPLACE_POS);
    }

    public BlockPos getHangout() {
        return get(HANGOUT_POS);
    }

    /**
     * Forces the villager's home to be set to their position. No checks for safety are made.
     * This is used on overwriting the original villager.
     */
    public void forcePositionAsHome() {
        this.home = this.func_180425_c();
    }

    private void setHome(EntityPlayerMP player) {
        if (func_184595_k(player.field_70165_t, player.field_70163_u, player.field_70161_v)) {
            say(Optional.of(player), "interaction.sethome.success");
            this.home = player.func_180425_c();
            this.func_175449_a(this.home, 32);
            BlockPos bed = searchBed();
            if (bed != null) {
                set(BED_POS, bed);
            }
        } else {
            say(Optional.of(player), "interaction.sethome.fail");
        }
    }

    public void setWorkplace(EntityPlayerMP player) {
        say(Optional.of(player), "interaction.setworkplace.success");
        set(WORKPLACE_POS, player.func_180425_c());
    }

    public void setHangout(EntityPlayerMP player) {
        say(Optional.of(player), "interaction.sethangout.success");
        set(HANGOUT_POS, player.func_180425_c());
    }

    public void say(Optional<EntityPlayer> player, String phraseId, @Nullable String... params) {
        ArrayList<String> paramsList = new ArrayList<>();
        if (params != null) Collections.addAll(paramsList, params);

        if (player.isPresent()) {
            EntityPlayer thePlayer = player.get();

            // Provide player as the first param, always
            paramsList.add(0, thePlayer.func_70005_c_());

            // Infected villagers do not speak.
            if (get(IS_INFECTED)) {
                thePlayer.func_145747_a(new TextComponentString(func_145748_c_().func_150254_d() + ": " + "???"));
                this.func_184185_a(SoundEvents.field_187899_gZ, 0.5F, field_70146_Z.nextFloat() + 0.5F);
            } else {
                String dialogueType = getPlayerHistoryFor(player.get().func_110124_au()).getDialogueType().getId();
                String phrase = MCA.getLocalizer().localize(dialogueType + "." + phraseId, paramsList);
                thePlayer.func_145747_a(new TextComponentString(String.format("%1$s: %2$s", func_145748_c_().func_150254_d(), phrase)));
            }
        } else {
            MCA.getLog().warn(new Throwable("Say called on player that is not present!"));
        }
    }

    public boolean isMarried() {
        return !get(SPOUSE_UUID).or(Constants.ZERO_UUID).equals(Constants.ZERO_UUID);
    }

    public boolean isMarriedTo(UUID uuid) {
        return get(SPOUSE_UUID).or(Constants.ZERO_UUID).equals(uuid);
    }

    public void marry(EntityPlayer player) {
        set(SPOUSE_UUID, Optional.of(player.func_110124_au()));
        set(SPOUSE_NAME, player.func_70005_c_());
        set(MARRIAGE_STATE, EnumMarriageState.MARRIED.getId());
    }

    private void endMarriage() {
        set(SPOUSE_UUID, Optional.of(Constants.ZERO_UUID));
        set(SPOUSE_NAME, "");
        set(MARRIAGE_STATE, EnumMarriageState.NOT_MARRIED.getId());
    }

    private void handleInteraction(EntityPlayerMP player, PlayerHistory history, APIButton button) {
        float successChance = 0.85F;
        int heartsBoost = button.getConstraints().contains(EnumConstraint.ADULTS) ? 15 : 5;

        String interactionName = button.getIdentifier().replace("gui.button.", "");

        successChance -= button.getConstraints().contains(EnumConstraint.ADULTS) ? 0.25F : 0.0F;
        successChance += (history.getHearts() / 10.0D) * 0.025F;

        if (MCA.getConfig().enableDiminishingReturns) successChance -= history.getInteractionFatigue() * 0.05F;

        boolean succeeded = field_70146_Z.nextFloat() < successChance;
        if (MCA.getConfig().enableDiminishingReturns && succeeded)
            heartsBoost -= history.getInteractionFatigue() * 0.05F;

        history.changeInteractionFatigue(1);
        history.changeHearts(succeeded ? heartsBoost : (heartsBoost * -1));
        String responseId = String.format("%s.%s", interactionName, succeeded ? "success" : "fail");
        say(Optional.of(player), responseId);
    }

    public void handleButtonClick(EntityPlayerMP player, String guiKey, String buttonId) {
        PlayerHistory history = getPlayerHistoryFor(player.func_110124_au());
        java.util.Optional<APIButton> button = API.getButtonById(guiKey, buttonId);
        if (!button.isPresent()) {
            MCA.getLog().warn("Button not found for key and ID: " + guiKey + ", " + buttonId);
        } else if (button.get().isInteraction()) handleInteraction(player, history, button.get());

        switch (buttonId) {
            case "gui.button.move":
                set(MOVE_STATE, EnumMoveState.MOVE.getId());
                this.playerToFollowUUID = Constants.ZERO_UUID;
                break;
            case "gui.button.stay":
                set(MOVE_STATE, EnumMoveState.STAY.getId());
                break;
            case "gui.button.follow":
                set(MOVE_STATE, EnumMoveState.FOLLOW.getId());
                this.playerToFollowUUID = player.func_110124_au();
                stopChore();
                break;
            case "gui.button.ridehorse":
                toggleMount(player);
                break;
            case "gui.button.sethome":
                setHome(player);
                break;
            case "gui.button.gohome":
                goHome(player);
                break;
            case "gui.button.setworkplace":
                setWorkplace(player);
                break;
            case "gui.button.sethangout":
                setHangout(player);
                break;
            case "gui.button.trade":
                if (MCA.getConfig().allowTrading) {
                    func_70932_a_(player);
                    player.func_180472_a(this);
                } else {
                    player.func_145747_a(new TextComponentString(MCA.getLocalizer().localize("info.trading.disabled")));
                }
                break;
            case "gui.button.inventory":
                player.openGui(MCA.getInstance(), Constants.GUI_ID_INVENTORY, player.field_70170_p, this.func_145782_y(), 0, 0);
                break;
            case "gui.button.gift":
                ItemStack stack = player.field_71071_by.func_70301_a(player.field_71071_by.field_70461_c);
                int giftValue = API.getGiftValueFromStack(stack);
                if (!handleSpecialCaseGift(player, stack)) {
                    if (stack.func_77973_b() == Items.field_151153_ao) set(IS_INFECTED, false);
                    else {
                        history.changeHearts(giftValue);
                        say(Optional.of(player), API.getResponseForGift(stack));
                    }
                }
                if (giftValue > 0) {
                    player.field_71071_by.func_70298_a(player.field_71071_by.field_70461_c, 1);
                }
                break;
            case "gui.button.procreate":
                if (PlayerSaveData.get(player).isBabyPresent())
                    say(Optional.of(player), "interaction.procreate.fail.hasbaby");
                else if (history.getHearts() < 100) say(Optional.of(player), "interaction.procreate.fail.lowhearts");
                else {
                    EntityAITasks.EntityAITaskEntry task = field_70714_bg.field_75782_a.stream().filter((ai) -> ai.field_75733_a instanceof EntityAIProcreate).findFirst().orElse(null);
                    if (task != null) {
                        ((EntityAIProcreate) task.field_75733_a).procreateTimer = 20 * 3; // 3 seconds
                        set(IS_PROCREATING, true);
                    }
                }
                break;
            case "gui.button.infected":
                set(IS_INFECTED, !get(IS_INFECTED));
                break;
            case "gui.button.texture.randomize":
                set(TEXTURE, API.getRandomSkin(this));
                break;
            case "gui.button.profession.randomize":
                setProfession(ProfessionsMCA.randomProfession());
                setVanillaCareer(getProfessionForge().getRandomCareer(field_70170_p.field_73012_v));
                break;
            case "gui.button.gender":
                EnumGender gender = EnumGender.byId(get(GENDER));
                if (gender == EnumGender.MALE) {
                    set(GENDER, EnumGender.FEMALE.getId());
                } else {
                    set(GENDER, EnumGender.MALE.getId());
                }
                // intentional fall-through here
            case "gui.button.texture":
                set(TEXTURE, API.getRandomSkin(this));
                break;
            case "gui.button.random":
                set(VILLAGER_NAME, API.getRandomName(EnumGender.byId(get(GENDER))));
                break;
            case "gui.button.profession":
                RegistryNamespaced<ResourceLocation, VillagerRegistry.VillagerProfession> registry = ObfuscationReflectionHelper.getPrivateValue(VillagerRegistry.class, VillagerRegistry.instance(), "REGISTRY");
                setProfession(ProfessionsMCA.randomProfession());
                setVanillaCareer(getProfessionForge().getRandomCareer(field_70170_p.field_73012_v));
                applySpecialAI();
                break;
            case "gui.button.prospecting":
                startChore(EnumChore.PROSPECT, player);
                break;
            case "gui.button.hunting":
                startChore(EnumChore.HUNT, player);
                break;
            case "gui.button.fishing":
                startChore(EnumChore.FISH, player);
                break;
            case "gui.button.chopping":
                startChore(EnumChore.CHOP, player);
                break;
            case "gui.button.harvesting":
                startChore(EnumChore.HARVEST, player);
                break;
            case "gui.button.stopworking":
                stopChore();
                break;
        }
    }

    private boolean handleSpecialCaseGift(EntityPlayer player, ItemStack stack) {
        Item item = stack.func_77973_b();

        if (item instanceof ItemSpecialCaseGift && !this.func_70631_g_()) { // special case gifts are rings so far so prevent giving them to children
            boolean decStackSize = ((ItemSpecialCaseGift) item).handle(player, this);
            if (decStackSize) player.field_71071_by.func_70298_a(player.field_71071_by.field_70461_c, -1);
            return true;
        } else if (item == Items.field_151105_aU) {
            Optional<Entity> spouse = Util.getEntityByUUID(field_70170_p, get(SPOUSE_UUID).or(Constants.ZERO_UUID));
            if (spouse.isPresent()) {
                EntityVillagerMCA progressor = this.get(GENDER) == EnumGender.FEMALE.getId() ? this : (EntityVillagerMCA) spouse.get();
                progressor.set(HAS_BABY, true);
                progressor.set(BABY_IS_MALE, field_70146_Z.nextBoolean());
                progressor.spawnParticles(EnumParticleTypes.HEART);
            } else say(Optional.of(player), "gift.cake.fail");
        } else if (item == Items.field_151153_ao && this.func_70631_g_()) {
            this.func_110195_a(((startingAge / 4) / 20 * -1));
            return true;
        }

        return false;
    }

    private void onEachClientUpdate() {
        if (get(IS_PROCREATING)) {
            this.field_70759_as += 50.0F;
        }

        if (this.field_70173_aa % 20 == 0) {
            onEachClientSecond();
        }
    }

    private void onEachClientSecond() {
        this.setSizeForAge();
    }

    private void onEachServerUpdate() {
        if (this.field_70173_aa % 20 == 0) { // Every second
            onEachServerSecond();
        }

        if (this.field_70173_aa % 200 == 0 && this.func_110143_aJ() > 0.0F) { // Every 10 seconds and when we're not already dead
            if (this.func_110143_aJ() < this.func_110138_aP()) {
                this.func_70606_j(this.func_110143_aJ() + 1.0F); // heal
            }
        }

        if (func_70631_g_()) {
            EnumAgeState current = EnumAgeState.byId(get(AGE_STATE));
            EnumAgeState target = EnumAgeState.byCurrentAge(startingAge, func_70874_b());
            if (current != target) {
                set(AGE_STATE, target.getId());
            }
        }
    }

    private void onEachServerSecond() {
        NBTTagCompound memories = get(PLAYER_HISTORY_MAP);
        memories.func_150296_c().forEach((key) -> PlayerHistory.fromNBT(this, UUID.fromString(key), memories.func_74775_l(key)).update());

        if (get(HAS_BABY)) {
            set(BABY_AGE, get(BABY_AGE) + 1);

            if (get(BABY_AGE) >= MCA.getConfig().babyGrowUpTime * 60) { // grow up time is in minutes and we measure age in seconds
                EntityVillagerMCA child = new EntityVillagerMCA(field_70170_p, Optional.absent(), Optional.of(get(BABY_IS_MALE) ? EnumGender.MALE : EnumGender.FEMALE));
                child.set(EntityVillagerMCA.AGE_STATE, EnumAgeState.BABY.getId());
                child.setStartingAge(MCA.getConfig().childGrowUpTime * 60 * 20 * -1);
                child.func_98054_a(true);
                child.func_70107_b(this.field_70165_t, this.field_70163_u, this.field_70161_v);
                child.set(EntityVillagerMCA.PARENTS, ParentData.create(this.func_110124_au(), this.get(SPOUSE_UUID).get(), this.get(VILLAGER_NAME), this.get(SPOUSE_NAME)).toNBT());
                field_70170_p.func_72838_d(child);

                set(HAS_BABY, false);
                set(BABY_AGE, 0);
            }
        }
    }

    public ResourceLocation getTextureResourceLocation() {
        if (get(IS_INFECTED)) {
            return ResourceLocationCache.getResourceLocationFor(String.format("mca:skins/%s/zombievillager.png", get(GENDER) == EnumGender.MALE.getId() ? "male" : "female"));
        } else {
            return ResourceLocationCache.getResourceLocationFor(get(TEXTURE));
        }
    }

    @Override
    protected void func_184651_r() {
        super.func_184651_r();
        this.field_70714_bg.func_75776_a(0, new EntityAIProspecting(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIHunting(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIChopping(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIHarvesting(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIFishing(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIMoveState(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIAgeBaby(this));
        this.field_70714_bg.func_75776_a(0, new EntityAIProcreate(this));
        this.field_70714_bg.func_75776_a(5, new EntityAIGoWorkplace(this));
        this.field_70714_bg.func_75776_a(5, new EntityAIGoHangout(this));
        this.field_70714_bg.func_75776_a(1, new EntityAISleeping(this));
        this.field_70714_bg.func_75776_a(10, new EntityAIWatchClosest(this, EntityPlayer.class, 8.0F));
        this.field_70714_bg.func_75776_a(10, new EntityAILookIdle(this));
    }

    private void applySpecialAI() {
        if (getProfessionForge() == ProfessionsMCA.bandit) {
            this.field_70714_bg.field_75782_a.clear();
            this.field_70714_bg.func_75776_a(1, new EntityAIAttackMelee(this, 0.8D, false));
            this.field_70714_bg.func_75776_a(2, new EntityAIMoveThroughVillage(this, 0.6D, false));

            this.field_70715_bh.func_75776_a(0, new EntityAINearestAttackableTarget<>(this, EntityVillagerMCA.class, 100, false, false, BANDIT_TARGET_SELECTOR));
            this.field_70715_bh.func_75776_a(1, new EntityAINearestAttackableTarget<>(this, EntityPlayer.class, true));
        } else if (getProfessionForge() == ProfessionsMCA.guard) {
            removeCertainTasks(EntityAIAvoidEntity.class);

            this.field_70714_bg.func_75776_a(1, new EntityAIAttackMelee(this, 0.8D, false));
            this.field_70714_bg.func_75776_a(2, new EntityAIMoveThroughVillage(this, 0.6D, false));

            this.field_70715_bh.func_75776_a(0, new EntityAINearestAttackableTarget<>(this, EntityVillagerMCA.class, 100, false, false, GUARD_TARGET_SELECTOR));
            this.field_70715_bh.func_75776_a(0, new EntityAINearestAttackableTarget<>(this, EntityZombie.class, 100, false, false, null));
            this.field_70715_bh.func_75776_a(0, new EntityAINearestAttackableTarget<>(this, EntityVex.class, 100, false, false, null));
            this.field_70715_bh.func_75776_a(0, new EntityAINearestAttackableTarget<>(this, EntityVindicator.class, 100, false, false, null));
        } else {
            //every other villager is allowed to defend itself from zombies while fleeing
            this.field_70714_bg.func_75776_a(0, new EntityAIDefendFromTarget(this));

            this.field_70715_bh.field_75782_a.clear();
            this.field_70715_bh.func_75776_a(0, new EntityAINearestAttackableTarget<>(this, EntityZombie.class, 100, false, false, null));
        }
    }

    //guards should not run away from zombies
    //TODO: should only avoid zombies when low on health
    private void removeCertainTasks(Class typ) {
        Iterator<EntityAITasks.EntityAITaskEntry> iterator = this.field_70714_bg.field_75782_a.iterator();

        while (iterator.hasNext()) {
            EntityAITasks.EntityAITaskEntry entityaitasks$entityaitaskentry = iterator.next();
            EntityAIBase entityaibase = entityaitasks$entityaitaskentry.field_75733_a;

            if (entityaibase.getClass().equals(typ)) {
                iterator.remove();
            }
        }
    }

    public void spawnParticles(EnumParticleTypes particleType) {
        if (this.field_70170_p.field_72995_K) {
            for (int i = 0; i < 5; ++i) {
                double d0 = this.field_70146_Z.nextGaussian() * 0.02D;
                double d1 = this.field_70146_Z.nextGaussian() * 0.02D;
                double d2 = this.field_70146_Z.nextGaussian() * 0.02D;
                this.field_70170_p.func_175688_a(particleType, this.field_70165_t + (double) (this.field_70146_Z.nextFloat() * this.field_70130_N * 2.0F) - (double) this.field_70130_N, this.field_70163_u + 1.0D + (double) (this.field_70146_Z.nextFloat() * this.field_70131_O), this.field_70161_v + (double) (this.field_70146_Z.nextFloat() * this.field_70130_N * 2.0F) - (double) this.field_70130_N, d0, d1, d2);
            }
        } else {
            NetMCA.INSTANCE.sendToAll(new NetMCA.SpawnParticles(this.func_110124_au(), particleType));
        }
    }

    public void stopChore() {
        set(ACTIVE_CHORE, EnumChore.NONE.getId());
        set(CHORE_ASSIGNING_PLAYER, Optional.of(Constants.ZERO_UUID));
    }

    public void startChore(EnumChore chore, EntityPlayer player) {
        set(ACTIVE_CHORE, chore.getId());
        set(CHORE_ASSIGNING_PLAYER, Optional.of(player.func_110124_au()));
    }

    public boolean playerIsParent(EntityPlayer player) {
        ParentData data = ParentData.fromNBT(get(PARENTS));
        return data.getParent1UUID().equals(player.func_110124_au()) || data.getParent2UUID().equals(player.func_110124_au());
    }

    @Override
    public BlockPos func_180486_cf() {
        return home;
    }

    @Override
    public void func_110177_bN() {
        // no-op, skip EntityVillager's detaching homes which messes up MoveTowardsRestriction.
    }

    public String getCurrentActivity() {
        EnumMoveState moveState = EnumMoveState.byId(get(MOVE_STATE));
        if (moveState != EnumMoveState.MOVE) {
            return moveState.getFriendlyName();
        }

        EnumChore chore = EnumChore.byId(get(ACTIVE_CHORE));
        if (chore != EnumChore.NONE) {
            return chore.getFriendlyName();
        }

        return null;
    }

    public void moveTowardsBlock(BlockPos target) {
        moveTowardsBlock(target, 0.5D);
    }

    public void moveTowardsBlock(BlockPos target, double speed) {
        double range = func_70661_as().func_111269_d() - 6.0D;

        if (func_174818_b(target) > Math.pow(range, 2.0)) {
            Vec3d vec3d = RandomPositionGenerator.func_75464_a(this, (int) range, 8, new Vec3d(target.func_177958_n(), target.func_177956_o(), target.func_177952_p()));
            if (vec3d != null && !func_70661_as().func_75484_a(func_70661_as().func_75488_a(vec3d.field_72450_a, vec3d.field_72448_b, vec3d.field_72449_c), speed)) {
                func_184595_k(vec3d.field_72450_a, vec3d.field_72448_b, vec3d.field_72449_c);
            }
        } else {
            if (!func_70661_as().func_75484_a(func_70661_as().func_179680_a(target), speed)) {
                func_184595_k(target.func_177958_n(), target.func_177956_o(), target.func_177952_p());
            }
        }
    }

    //searches for the nearest bed
    public BlockPos searchBed() {
        List<BlockPos> nearbyBeds = Util.getNearbyBlocks(func_190671_u_(), field_70170_p, BlockBed.class, 8, 8);
        List<BlockPos> valid = new ArrayList<>();
        for (BlockPos pos : nearbyBeds) {
            IBlockState state = field_70170_p.func_180495_p(pos);
            if (!(state.func_177229_b(field_176471_b)) && state.func_177229_b(field_176472_a) != BlockBed.EnumPartType.HEAD) {
                valid.add(pos);
            }
        }
        return Util.getNearestPoint(func_190671_u_(), valid);
    }

    /**
     * Returns the orientation of the bed in degrees.
     */
    @SideOnly(Side.CLIENT)
    public float getBedOrientationInDegrees() {
        BlockPos bedLocation = get(EntityVillagerMCA.BED_POS);
        IBlockState state = bedLocation == BlockPos.field_177992_a ? null : this.field_70170_p.func_180495_p(bedLocation);
        if (state != null && state.func_177230_c().isBed(state, field_70170_p, bedLocation, this)) {
            EnumFacing enumfacing = state.func_177230_c().getBedDirection(state, field_70170_p, bedLocation);

            switch (enumfacing) {
                case SOUTH:
                    return 90.0F;
                case WEST:
                    return 0.0F;
                case NORTH:
                    return 270.0F;
                case EAST:
                    return 180.0F;
            }
        }

        return 0.0F;
    }

    public boolean isSleeping() {
        return get(SLEEPING);
    }

    private void updateSleeping() {
        if (isSleeping()) {
            BlockPos bedLocation = get(EntityVillagerMCA.BED_POS);

            final IBlockState state = this.field_70170_p.func_175667_e(bedLocation) ? this.field_70170_p.func_180495_p(bedLocation) : null;
            final boolean isBed = state != null && state.func_177230_c().isBed(state, this.field_70170_p, bedLocation, this);

            if (isBed) {
                final EnumFacing enumfacing = state.func_177230_c() instanceof BlockHorizontal ? state.func_177229_b(BlockHorizontal.field_185512_D) : null;

                if (enumfacing != null) {
                    float f1 = 0.5F + (float) enumfacing.func_82601_c() * 0.4F;
                    float f = 0.5F + (float) enumfacing.func_82599_e() * 0.4F;
                    this.setRenderOffsetForSleep(enumfacing);
                    this.func_70107_b((double) ((float) bedLocation.func_177958_n() + f1), (double) ((float) bedLocation.func_177956_o() + 0.6875F), (double) ((float) bedLocation.func_177952_p() + f));
                } else {
                    this.func_70107_b((double) ((float) bedLocation.func_177958_n() + 0.5F), (double) ((float) bedLocation.func_177956_o() + 0.6875F), (double) ((float) bedLocation.func_177952_p() + 0.5F));
                }

                this.func_70105_a(0.2F, 0.2F);

                this.field_70159_w = 0.0D;
                this.field_70181_x = 0.0D;
                this.field_70179_y = 0.0D;
            } else {
                set(EntityVillagerMCA.BED_POS, BlockPos.field_177992_a);
                stopSleeping();
            }
        } else {
            this.func_70105_a(0.6F, 1.8F);
        }
    }

    private void setRenderOffsetForSleep(EnumFacing bedDirection) {
        this.renderOffsetX = -1.0F * (float) bedDirection.func_82601_c();
        this.renderOffsetZ = -1.0F * (float) bedDirection.func_82599_e();
    }

    public void startSleeping() {
        if (this.func_184218_aH()) {
            this.func_184210_p();
        }

        set(SLEEPING, true);

        BlockPos bedLocation = get(EntityVillagerMCA.BED_POS);
        IBlockState blockstate = this.field_70170_p.func_180495_p(bedLocation);
        if (blockstate.func_177230_c() == Blocks.field_150324_C) {
            blockstate.func_177230_c().setBedOccupied(field_70170_p, bedLocation, null, true);
        }
    }

    public void stopSleeping() {
        BlockPos bedLocation = get(EntityVillagerMCA.BED_POS);
        if (bedLocation != BlockPos.field_177992_a) {
            IBlockState blockstate = this.field_70170_p.func_180495_p(bedLocation);

            if (blockstate.func_177230_c().isBed(blockstate, field_70170_p, bedLocation, this)) {
                blockstate.func_177230_c().setBedOccupied(field_70170_p, bedLocation, null, false);
                BlockPos blockpos = blockstate.func_177230_c().getBedSpawnPosition(blockstate, field_70170_p, bedLocation, null);

                if (blockpos == null) {
                    blockpos = bedLocation.func_177984_a();
                }

                this.func_70107_b((double) ((float) blockpos.func_177958_n() + 0.5F), (double) ((float) blockpos.func_177956_o() + 0.1F), (double) ((float) blockpos.func_177952_p() + 0.5F));
            }
        }

        set(SLEEPING, false);
    }
}
