package mca.entity.data;

import lombok.Getter;
import mca.core.Constants;
import mca.entity.EntityVillagerMCA;
import mca.enums.EnumDialogueType;
import net.minecraft.nbt.NBTTagCompound;
import java.util.UUID;


import java.util.UUID;

public class PlayerHistory {
    @Getter private int hearts;
    @Getter private int interactionFatigue;
    @Getter private boolean giftPresent;
    @Getter private int greetTimer;
    @Getter private EnumDialogueType dialogueType;

    @Getter private UUID playerUUID;
    private EntityVillagerMCA villager;

    private PlayerHistory() {
        hearts = 0;
        interactionFatigue = 0;
        giftPresent = false;
        greetTimer = 0;
        playerUUID = Constants.ZERO_UUID;
        dialogueType = EnumDialogueType.ADULT;
    }

    public static PlayerHistory getNew(EntityVillagerMCA villager, UUID uuid) {
        PlayerHistory history = new PlayerHistory();
        history.villager = villager;
        history.playerUUID = uuid;

        if (villager.func_70631_g_()) {
            history.setDialogueType(EnumDialogueType.CHILD);
        } else {
            history.setDialogueType(EnumDialogueType.ADULT);
        }
        return history;
    }

    public static PlayerHistory fromNBT(EntityVillagerMCA villager, UUID uuid, NBTTagCompound nbt) {
        PlayerHistory history = new PlayerHistory();
        history.villager = villager;
        history.playerUUID = uuid;

        history.hearts = nbt.func_74762_e("hearts");
        history.interactionFatigue = nbt.func_74762_e("interactionFatigue");
        history.giftPresent = nbt.func_74767_n("giftPresent");
        history.greetTimer = nbt.func_74762_e("greetTimer");
        history.dialogueType = EnumDialogueType.byValue(nbt.func_74779_i("dialogueType"));

        return history;
    }

    public NBTTagCompound toNBT() {
        NBTTagCompound nbt = new NBTTagCompound();

        nbt.func_74768_a("hearts", hearts);
        nbt.func_74768_a("interactionFatigue", interactionFatigue);
        nbt.func_74757_a("giftPresent", giftPresent);
        nbt.func_74768_a("greetTimer", greetTimer);
        nbt.func_74778_a("dialogueType", dialogueType.getId());

        return nbt;
    }

    public void setHearts(int value) {
        hearts = value;
        villager.updatePlayerHistoryMap(this);
    }

    public void changeHearts(int value) {
        hearts += value;
        villager.updatePlayerHistoryMap(this);
    }

    public void changeInteractionFatigue(int value) {
        interactionFatigue += value;
        villager.updatePlayerHistoryMap(this);
    }

    public void update() {
        // every 5 minutes reduce interaction fatigues
        if (villager.field_70173_aa % 6000 == 0) changeInteractionFatigue(-1);
    }

    public void setDialogueType(EnumDialogueType type) {
        this.dialogueType = type;
        villager.updatePlayerHistoryMap(this);
    }
}
