package mca.entity.inventory;

import mca.entity.EntityVillagerMCA;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.inventory.InventoryBasic;
import net.minecraft.item.Item;
import net.minecraft.item.ItemArmor;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Nullable;


import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class InventoryMCA extends InventoryBasic {
    private EntityVillagerMCA villager;

    public InventoryMCA(EntityVillagerMCA villager) {
        super("Villager Inventory", true, 27);
        this.villager = villager;
    }

    public int getFirstSlotContainingItem(Item item) {
        for (int i = 0; i < this.func_70302_i_(); i++) {
            ItemStack stack = this.func_70301_a(i);
            if (stack.func_77973_b() != item) continue;
            return i;
        }
        return -1;
    }

    public boolean contains(Class clazz) {
        for (int i = 0; i < this.func_70302_i_(); ++i) {
            final ItemStack stack = this.func_70301_a(i);
            final Item item = stack.func_77973_b();

            if (item.getClass() == clazz) return true;
        }
        return false;
    }

    /**
     * Gets the best quality (max damage) item of the specified type that is in the inventory.
     *
     * @param type The class of item that will be returned.
     * @return The item stack containing the item of the specified type with the highest max damage.
     */
    public ItemStack getBestItemOfType(@Nullable Class type) {
        if (type == null) return ItemStack.field_190927_a;
        else return func_70301_a(getBestItemOfTypeSlot(type));
    }

    public ItemStack getBestArmorOfType(EntityEquipmentSlot slot) {
        ItemStack returnStack = ItemStack.field_190927_a;

        List<ItemStack> armors = new ArrayList();
        for (int i = 0; i < this.func_70302_i_(); ++i) {
            ItemStack stack = this.func_70301_a(i);
            if (stack.func_77973_b() instanceof ItemArmor) {
                ItemArmor armor = (ItemArmor) stack.func_77973_b();
                if (armor.field_77881_a == slot) armors.add(stack);
            }
        }

        int highestMaxDamage = 0;
        for (ItemStack stack : armors) {
            if (stack.func_77958_k() > highestMaxDamage) {
                returnStack = stack;
                highestMaxDamage = stack.func_77958_k();
            }
        }
        return returnStack;
    }

    public int getBestItemOfTypeSlot(Class type) {
        int highestMaxDamage = 0;
        int best = -1;

        for (int i = 0; i < this.func_70302_i_(); ++i) {
            ItemStack stackInInventory = this.func_70301_a(i);

            final String itemClassName = stackInInventory.func_77973_b().getClass().getName();

            if (itemClassName.equals(type.getName()) && highestMaxDamage < stackInInventory.func_77958_k()) {
                highestMaxDamage = stackInInventory.func_77958_k();
                best = i;
            }
        }

        return best;
    }

    public void dropAllItems() {
        for (int i = 0; i < this.func_70302_i_(); i++) {
            ItemStack stack = this.func_70301_a(i);
            villager.func_70099_a(stack, 1.0F);
        }
        func_174888_l();
    }

    public void readInventoryFromNBT(NBTTagList tagList) {
        for (int i = 0; i < this.func_70302_i_(); ++i) {
            this.func_70299_a(i, ItemStack.field_190927_a);
        }

        for (int i = 0; i < tagList.func_74745_c(); ++i) {
            NBTTagCompound nbt = tagList.func_150305_b(i);
            int slot = nbt.func_74771_c("Slot") & 255;

            if (slot < this.func_70302_i_()) {
                this.func_70299_a(slot, new ItemStack(nbt));
            }
        }
    }

    public NBTTagList writeInventoryToNBT() {
        NBTTagList tagList = new NBTTagList();

        for (int i = 0; i < this.func_70302_i_(); ++i) {
            ItemStack itemstack = this.func_70301_a(i);

            if (itemstack != ItemStack.field_190927_a) {
                NBTTagCompound nbt = new NBTTagCompound();
                nbt.func_74774_a("Slot", (byte) i);
                itemstack.func_77955_b(nbt);
                tagList.func_74742_a(nbt);
            }
        }

        return tagList;
    }
}
